import React, { FC, useEffect, useState } from 'react';
import { View } from 'react-native';
import { SetDimmer } from '../switchAndDimmer/components/setDimmer';
import { SetSwitch } from '../switchAndDimmer/components/setSwitch';
import { RGBDimmerWidgetDetailModel } from '../widgetDetailModels';
import { ColorWheelPicker } from './components/colorWheelPicker';
import { styles } from './styles';

const booleanValues = {
  MAX: 100,
  MIN: 0,
};

const valueFromBoolean = {
  true: booleanValues.MAX,
  false: booleanValues.MIN,
};

const DEFAULT_COLOR = '#FFFFFF';

const getValidatedHexColor = (color: string): string => {
  const hexColorRegex = /^#(?:[0-9a-f]{3}){1,2}$/i;
  if (hexColorRegex.test(color)) {
    return color;
  }
  return DEFAULT_COLOR;
};

const getRoundedValue = (value: string | number): number => {
  return typeof value === 'string' ? Math.round(Number(value)) : Math.round(value);
};

export const RGBDimmerWidgetDetail: FC<RGBDimmerWidgetDetailModel> = ({
  id,
  dimmerValue = booleanValues.MIN,
  colorPickerValue = DEFAULT_COLOR,
  title,
  colorSampleTitle,
  sliderTitle,
  subtitle,
  loadingInfoTextOptions,
  onUpdateData,
  loading = false,
}) => {
  const [state, setState] = useState<number>(getRoundedValue(dimmerValue));
  const [isEnabled, setIsEnabled] = useState<boolean>(!!getRoundedValue(dimmerValue));
  const [intensity, setIntensity] = useState<number>(getRoundedValue(dimmerValue));
  
  useEffect(() => {
    setIsEnabled(dimmerValue !== booleanValues.MIN);
    setState(getRoundedValue(dimmerValue));
  }, [dimmerValue]);

  useEffect(() => {
    onUpdateData?.({ id, dimmerValue: intensity, hexColorValue: colorPickerValue });
  }, [intensity]);

  const handleOnFinishDimmer = (intensity: number) => {
    const editValueEnabled = !!intensity;
    setIsEnabled(editValueEnabled);
    setIntensity(intensity);
    setState(intensity);
  };
 
  const handleOnChangeSwitchValue = (response: boolean) => {
    const editValue = valueFromBoolean[`${response}`];
    setIsEnabled(response);
    setIntensity(editValue);
    setState(editValue);
  };
  
  const handleOnChangeColorWheel = (color: string) => {
    onUpdateData?.({ id, dimmerValue: intensity, hexColorValue: color });
  };

  return (
    <View>
      <View style={styles.setSwitchContainer}>
        <SetSwitch
          title={title}
          subtitle={subtitle}
          loadingInfoTextOptions={loadingInfoTextOptions}
          value={!!state}
          onValueChange={handleOnChangeSwitchValue}
          loading={loading}
          enabled={isEnabled}
        />
      </View>
      <View style={styles.colorWheelPickerContainer}>
        <ColorWheelPicker 
          color={getValidatedHexColor(colorPickerValue)}
          colorSampleTitle={colorSampleTitle}
          onColorChangeComplete={handleOnChangeColorWheel}
          loading={loading}
          enabled={isEnabled}
        />
      </View>
      <SetDimmer
        defaultValue={intensity}
        onFinishChange={handleOnFinishDimmer}
        loading={loading}
        sliderTitle={sliderTitle}
      />
    </View>
  );
};
